@@ -1,10 +1,10 @@
 /* Execute a C# program.
-   Copyright (C) 2003-2020 Free Software Foundation, Inc.
+   Copyright (C) 2003-2023 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -100,13 +100,14 @@ execute_csharp_using_mono (const char *assembly_path,
     {
       /* Test for presence of mono:
          "mono --version >/dev/null 2>/dev/null"  */
-      char *argv[3];
+      const char *argv[3];
       int exitstatus;
 
       argv[0] = "mono";
       argv[1] = "--version";
       argv[2] = NULL;
-      exitstatus = execute ("mono", "mono", argv, false, false, true, true,
+      exitstatus = execute ("mono", "mono", argv, NULL,
+                            false, false, true, true,
                             true, false, NULL);
       mono_present = (exitstatus == 0);
       mono_tested = true;
@@ -115,7 +116,8 @@ execute_csharp_using_mono (const char *assembly_path,
   if (mono_present)
     {
       char *old_monopath;
-      char **argv = (char **) xmalloca ((2 + nargs + 1) * sizeof (char *));
+      const char **argv =
+        (const char **) xmalloca ((2 + nargs + 1) * sizeof (const char *));
       unsigned int i;
       bool err;
 
@@ -123,9 +125,9 @@ execute_csharp_using_mono (const char *assembly_path,
       old_monopath = set_monopath (libdirs, libdirs_count, false, verbose);
 
       argv[0] = "mono";
-      argv[1] = (char *) assembly_path;
+      argv[1] = assembly_path;
       for (i = 0; i <= nargs; i++)
-        argv[2 + i] = (char *) args[i];
+        argv[2 + i] = args[i];
 
       if (verbose)
         {
@@ -162,12 +164,13 @@ execute_csharp_using_sscli (const char *assembly_path,
     {
       /* Test for presence of clix:
          "clix >/dev/null 2>/dev/null ; test $? = 1"  */
-      char *argv[2];
+      const char *argv[2];
       int exitstatus;
 
       argv[0] = "clix";
       argv[1] = NULL;
-      exitstatus = execute ("clix", "clix", argv, false, false, true, true,
+      exitstatus = execute ("clix", "clix", argv, NULL,
+                            false, false, true, true,
                             true, false, NULL);
       clix_present = (exitstatus == 0 || exitstatus == 1);
       clix_tested = true;
@@ -176,7 +179,8 @@ execute_csharp_using_sscli (const char *assembly_path,
   if (clix_present)
     {
       char *old_clixpath;
-      char **argv = (char **) xmalloca ((2 + nargs + 1) * sizeof (char *));
+      const char **argv =
+        (const char **) xmalloca ((2 + nargs + 1) * sizeof (const char *));
       unsigned int i;
       bool err;
 
@@ -184,9 +188,9 @@ execute_csharp_using_sscli (const char *assembly_path,
       old_clixpath = set_clixpath (libdirs, libdirs_count, false, verbose);
 
       argv[0] = "clix";
-      argv[1] = (char *) assembly_path;
+      argv[1] = assembly_path;
       for (i = 0; i <= nargs; i++)
-        argv[2 + i] = (char *) args[i];
+        argv[2 + i] = args[i];
 
       if (verbose)
         {
